﻿<%@ Control Language="C#" Inherits="System.Web.Mvc.ViewUserControl<dynamic>" %>
<div id="Drawing">
    <div id="Instructions">
        <p>
            This map demonstrates the drawing of items on the map. You can select items to show
            from <b>SQL Tables</b>. You can change the opacity of the items using the slider
            near the checkbox.
        </p>
        <p>
            The query type is changed using the selection from <b>Query Filter</b>. You can
            get items that are near a point using <b>Proximity</b>. You can get items that are
            near a polyline (i.e. a set of connected line segments) using <b>Line Buffer</b>.
            In both cases, the distance from the feature is set using the <b>Distance</b> slider.
            You can get items within a polygon using <b>Area of Interest</b>. Construct polylines
            and polygons by clicking on the map and double-clicking at the last point to finish
            the figure. Using <b>Viewport</b> mode will get all items within the visible area
            of the map. Click <b>Query</b> to get the new set of items.
        </p>
        <p>
            The current mouse location and the map zoom level can be seen in <b>Current View</b>.
        </p>
        <p>
            Metrics for the latest query are shown in <b>Metrics</b>. Among other items such
            as timing, this shows the number of features (e.g. Earthquakes or Counties) and
            the total number of points in all vertices.
        </p>
        <p>
            As the number of items being drawn on the map increases, the performance of the
            map decreases. For example, after drawing a few thousand faults or earthquakes on
            the map, panning the map becomes slow. If you need to display a number of items
            on a map, consider the use of Tiling.
        </p>
        <p>
            Another limitation on the number of items, of course, is the size of the data being
            sent to the client. It is usually sent in a single big chunk which gets larger as
            the number of items increases.
        </p>
        <p>
            <b>NOTE:</b> if you close this window you can show it again by clicking <b>Show Instructions</b>.
        </p>
    </div>
    <div id="MapPanel">
        <div id="ControlPanel">
            <div id="ShowItems" class="panel">
                <h3>
                    <a href="#">SQL Tables</a>
                </h3>
                <div class="contents">
                    <div class="headings">
                        <div class="left">
                            Table
                        </div>
                        <div class="right">
                            Opacity
                        </div>
                    </div>
                    <div class="item_row">
                        <div class="left">
                            <label>
                                <input type="checkbox" name="Countries" />
                                Countries
                            </label>
                        </div>
                        <div class="right">
                            <div class="opacity_slider">
                            </div>
                        </div>
                    </div>
                    <div class="item_row">
                        <div class="left">
                            <label>
                                <input type="checkbox" name="States" />
                                States
                            </label>
                        </div>
                        <div class="right">
                            <div class="opacity_slider">
                            </div>
                        </div>
                    </div>
                    <div class="item_row">
                        <div class="left">
                            <label>
                                <input type="checkbox" name="Counties" />
                                US Counties
                            </label>
                        </div>
                        <div class="right">
                            <div class="opacity_slider">
                            </div>
                        </div>
                    </div>
                    <div class="item_row">
                        <div class="left">
                            <label>
                                <input type="checkbox" name="Earthquakes" />
                                Earthquakes
                            </label>
                        </div>
                        <!-- no slider for pushpins since we cannot change opacity -->
                    </div>
                    <div class="item_row">
                        <div class="left">
                            <label>
                                <input type="checkbox" name="Faults" />
                                Faults
                            </label>
                        </div>
                        <div class="right">
                            <div class="opacity_slider">
                            </div>
                        </div>
                    </div>
                </div>
            </div>
            <br />
            <div id="QueryFilter" class="panel">
                <h3>
                    <a href="#">Query Filter</a>
                </h3>
                <div class="contents">
                    <div>
                        <label>
                            <input type="radio" name="QueryType" value="Proximity" checked="checked" />
                            Proximity
                        </label>
                    </div>
                    <div>
                        <label>
                            <input type="radio" name="QueryType" value="Line" />
                            Line Buffer
                        </label>
                    </div>
                    <div>
                        <div>
                            <div>
                                Distance: <span id="DistanceText"></span>Kilometers
                            </div>
                            <div id="DistanceSlider">
                            </div>
                        </div>
                    </div>
                    <div>
                        <label>
                            <input type="radio" name="QueryType" value="Area" />
                            Area of Interest
                        </label>
                    </div>
                    <div>
                        <label>
                            <input type="radio" name="QueryType" value="Viewport" />
                            Viewport
                        </label>
                    </div>
                </div>
            </div>
            <div id="QueryBox">
                <button id="QueryButton">
                    Query
                </button>
                <button id="ResetButton">
                    Reset
                </button>
                <button id="RedrawButton">
                    Redraw
                </button>
            </div>
            <div id="CurrentView" class="panel">
                <h3>
                    <a href="#">Current View</a>
                </h3>
                <div class="contents">
                    <div class="item_row">
                        <div class="left">
                            Latitude:
                        </div>
                        <div class="right" id="CurrentViewLatitude">
                        </div>
                    </div>
                    <div class="item_row">
                        <div class="left">
                            Longitude:
                        </div>
                        <div class="right" id="CurrentViewLongitude">
                        </div>
                    </div>
                    <div class="item_row">
                        <div class="left">
                            Zoom Level:
                        </div>
                        <div class="right" id="CurrentViewZoom">
                        </div>
                    </div>
                </div>
            </div>
            <br />
            <div id="Metrics" class="panel">
                <h3>
                    <a href="#">Metrics</a>
                </h3>
                <div class="contents">
                    <div class="item_row">
                        <div class="left">
                            Features:
                        </div>
                        <div class="right" id="MetricsNumberOfFeatures">
                        </div>
                    </div>
                    <div class="item_row">
                        <div class="left">
                            Points:
                        </div>
                        <div class="right" id="MetricsNumberOfPoints">
                        </div>
                    </div>
                    <div class="item_row">
                        <div class="left">
                            Response Size:
                        </div>
                        <div class="right" id="MetricsResponseSize">
                        </div>
                    </div>
                    <div class="item_row">
                        <div class="left">
                            Total Load Time:
                        </div>
                        <div class="right" id="MetricsTotalLoadTime">
                        </div>
                    </div>
                    <div class="item_row">
                        <div class="left">
                            Query Time:
                        </div>
                        <div class="right" id="MetricsQueryTimeMilliseconds">
                        </div>
                    </div>
                    <div class="item_row">
                        <div class="left">
                            Transmit Time:
                        </div>
                        <div class="right" id="MetricsTransmitTime">
                        </div>
                    </div>
                    <div class="item_row">
                        <div class="left">
                            Render Time:
                        </div>
                        <div class="right" id="MetricsRenderTime">
                        </div>
                    </div>
                </div>
            </div>
        </div>
        <div id="Map">
        </div>
        <button id="ShowControlPanel">
            Show Controls</button>
        <button id="ShowInstructions">
            Show Instructions</button>
        <table id="ItemProperties">
            <tbody>
            </tbody>
        </table>
    </div>
</div>
